@@ -131,7 +131,7 @@ export_add(nfs_export *exp)
 		xlog(L_FATAL, "unknown client type in export_add");
 
 	epp = exportlist + type;
-	while (*epp && slen < strlen((*epp)->m_export.e_path))
+	while (*epp && slen <= strlen((*epp)->m_export.e_path))
 		epp = &((*epp)->m_next);
 	exp->m_next = *epp;
 	*epp = exp;
@@ -240,6 +240,8 @@ export_freeall(void)
 				xfree(exp->m_export.e_sqgids);
 			if (exp->m_export.e_mountpoint)
 				free(exp->m_export.e_mountpoint);
+			if (exp->m_export.e_fslocdata)
+				xfree(exp->m_export.e_fslocdata);
 			xfree(exp);
 		}
 		exportlist[i] = NULL;
