@@ -19,9 +19,17 @@ enum {
 	MCL_WILDCARD,
 	MCL_NETGROUP,
 	MCL_ANONYMOUS,
+	MCL_GSS,
 	MCL_MAXTYPES
 };
 
+enum {
+	FSLOC_NONE = 0,
+	FSLOC_REFER,
+	FSLOC_REPLICA,
+	FSLOC_STUB
+};
+
 typedef struct mclient {
 	struct mclient *	m_next;
 	char			m_hostname[NFSCLNT_IDMAX+1];
@@ -54,13 +62,15 @@ int				client_check(nfs_client *, struct hostent *);
 int				client_match(nfs_client *, char *hname);
 void				client_release(nfs_client *);
 void				client_freeall(void);
+char *				client_compose(struct in_addr addr);
+int 				client_member(char *client, char *name);
 
 int				export_read(char *fname);
 void				export_add(nfs_export *);
 void				export_reset(nfs_export *);
 nfs_export *			export_lookup(char *hname, char *path, int caconical);
 nfs_export *			export_find(struct hostent *, char *path);
-struct exportent *		export_allowed(struct hostent *, char *path);
+nfs_export *			export_allowed(struct hostent *, char *path);
 nfs_export *			export_create(struct exportent *, int canonical);
 nfs_export *			export_dup(nfs_export *, struct hostent *);
 void				export_freeall(void);
